# [Operations Research](./https://github.com/YashBansod/Operations-Research/)  
MATLAB implementation of all the Operations Research algorithms and problems solved by me.  
Operations research (OR) is an analytical method of problem-solving and decision-making that is useful in the management of organizations. In operations research, problems are broken down into basic components and then solved in defined steps by mathematical analysis.

<div><span class="image fit"><img src="./operations_research.svg"></span></div>

## Project Contents

### [Linear Programming](./Linear_Programming/)

Linear programming (LP, also called linear optimization) is a method to achieve the best outcome (such as maximum profit or lowest cost) in a [mathematical model](https://en.wikipedia.org/wiki/Mathematical_model) whose requirements are represented by [linear relationships](https://en.wikipedia.org/wiki/Linear_function#As_a_polynomial_function). Linear programming is a special case of mathematical programming (also known as [mathematical optimization](https://en.wikipedia.org/wiki/Mathematical_optimization)).



### [Networks & Integer Programming](./Networks_and_Integer_Programming/)

An integer programming problem is a [mathematical optimization](https://en.wikipedia.org/wiki/Mathematical_optimization) or [feasibility](https://en.wikipedia.org/wiki/Constraint_satisfaction_problem) program in which some or all of the variables are restricted to be [integers](https://en.wikipedia.org/wiki/Integer). In many settings the term refers to integer [linear programming](https://en.wikipedia.org/wiki/Linear_programming) (ILP), in which the objective function and the constraints (other than the integer constraints) are [linear](https://en.wikipedia.org/wiki/Linear_function_(calculus)).

In this project, integer programming is used to solve network problems such as shortest path problem, transportation problem, transshipment problem, critical path problem etc.  



### [Monte Carlo Simulation](./Monte_Carlo_Simulation/)

Monte Carlo methods, or Monte Carlo experiments, are a broad class of [computational](https://en.wikipedia.org/wiki/Computation) [algorithms](https://en.wikipedia.org/wiki/Algorithm) that rely on repeated [random sampling](https://en.wikipedia.org/wiki/Random_sampling) to obtain numerical results. The underlying concept is to use [randomness](https://en.wikipedia.org/wiki/Randomness) to solve problems that might be [deterministic](https://en.wikipedia.org/wiki/Deterministic_system) in principle. They are often used in [physical](https://en.wikipedia.org/wiki/Physics) and [mathematical](https://en.wikipedia.org/wiki/Mathematics) problems and are most useful when it is difficult or impossible to use other approaches. Monte Carlo methods are mainly used in three problem classes:[optimization](https://en.wikipedia.org/wiki/Optimization), [numerical integration](https://en.wikipedia.org/wiki/Numerical_integration), and generating draws from a [probability distribution](https://en.wikipedia.org/wiki/Probability_distribution).



### [Markov Processes](./Markov_Processes/)

A Markov chain is a [stochastic model](https://en.wikipedia.org/wiki/Stochastic_model) describing a [sequence](https://en.wikipedia.org/wiki/Sequence) of possible events in which the probability of each event depends only on the state attained in the previous event. In [continuous-time](https://en.wikipedia.org/wiki/Continuous-time), it is known as a Markov process.**** It is named after the [Russian](https://en.wikipedia.org/wiki/Russia) mathematician [Andrey Markov](https://en.wikipedia.org/wiki/Andrey_Markov).

Markov chains have many applications as [statistical models](https://en.wikipedia.org/wiki/Statistical_model) of real-world processes, such as studying [cruise control systems](https://en.wikipedia.org/wiki/Cruise_control) in [motor vehicles](https://en.wikipedia.org/wiki/Motor_vehicle), queues or lines of customers arriving at an airport, currency [exchange rates](https://en.wikipedia.org/wiki/Exchange_rate) and animal population dynamics.

Markov processes are the basis for general stochastic simulation methods known as [Markov chain Monte Carlo](https://en.wikipedia.org/wiki/Markov_chain_Monte_Carlo), which are used for simulating sampling from complex probability distributions, and have found application in [Bayesian statistics](https://en.wikipedia.org/wiki/Bayesian_statistics) and [artificial intelligence](https://en.wikipedia.org/wiki/Artificial_intelligence).



### [Queueing Theory](./Queueing_Theory/)

Queueing theory is the mathematical study of waiting lines, or [queues](https://en.wiktionary.org/wiki/queue). A queueing model is constructed so that queue lengths and waiting time can be predicted. Queueing theory is generally considered a branch of [operations research](https://en.wikipedia.org/wiki/Operations_research) because the results are often used when making business decisions about the resources needed to provide a service.



### [Artificial Intelligence](./Artificial_Intelligence)

In [computer science](https://en.wikipedia.org/wiki/Computer_science), artificial intelligence(AI), sometimes called machine intelligence, is [intelligence](https://en.wikipedia.org/wiki/Intelligence) demonstrated by [machines](https://en.wikipedia.org/wiki/Machine), unlike the natural intelligence displayed by [humans](https://en.wikipedia.org/wiki/Human_intelligence) and [animals](https://en.wikipedia.org/wiki/Animal_cognition). Leading AI textbooks define the field as the study of "[intelligent agents](https://en.wikipedia.org/wiki/Intelligent_agent)": any device that perceives its environment and takes actions that maximize its chance of successfully achieving its goals. Colloquially, the term "artificial intelligence" is often used to describe machines (or computers) that mimic "cognitive" functions that humans associate with the [human mind](https://en.wikipedia.org/wiki/Human_mind), such as "learning" and "problem solving".



### [Kalman Filter](./Kalman_Filter/)

In [statistics](https://en.wikipedia.org/wiki/Statistics) and [control theory](https://en.wikipedia.org/wiki/Control_theory), Kalman filtering, also known as linear quadratic estimation (LQE), is an [algorithm](https://en.wikipedia.org/wiki/Algorithm) that uses a series of measurements observed over time, containing [statistical noise](https://en.wikipedia.org/wiki/Statistical_noise) and other inaccuracies, and produces estimates of unknown variables that tend to be more accurate than those based on a single measurement alone, by estimating a [joint probability distribution](https://en.wikipedia.org/wiki/Joint_probability_distribution) over the variables for each timeframe. The filter is named after [Rudolf E. Kálmán](https://en.wikipedia.org/wiki/Rudolf_E._Kálmán), one of the primary developers of its theory.



### [Linear Quadratic Regulator](./Linear_Quadratic_Regulator/)

The theory of [optimal control](https://en.wikipedia.org/wiki/Optimal_control) is concerned with operating a [dynamic system](https://en.wikipedia.org/wiki/Dynamic_system) at minimum cost.  The case where the system dynamics are described by a set of [linear differential equations](https://en.wikipedia.org/wiki/Linear_differential_equation) and the cost is described by a [quadratic](https://en.wikipedia.org/wiki/Quadratic_polynomial) [function](https://en.wikipedia.org/wiki/Functional_(mathematics)) is called the LQ problem.  One of the main results in the theory is that the solution is provided by the linear–quadratic regulator (LQR), a feedback controller whose equations are given below. The LQR is an important part of the solution to the [LQG (linear–quadratic–Gaussian) problem](https://en.wikipedia.org/wiki/Linear-quadratic-Gaussian_control). Like the LQR problem itself, the LQG problem is one of the most fundamental problems in [control theory](https://en.wikipedia.org/wiki/Control_theory).









